@@ -24,22 +24,16 @@ waitUser.default <- function(current.row, e){
 
 waitUser.text_question <- function(current.row, e){
   e$val <- str_trim(unlist(strsplit(readline("ANSWER: "),",")))
-#   e$row <- 1 + e$row
-#   e$iptr <- 1
   e$iptr <- 1 + e$iptr
 }
 
 waitUser.text_many_question <- function(current.row, e){
   e$val <- str_trim(unlist(strsplit(readline("ANSWER: "),",")))
-#   e$row <- 1 + e$row
-#   e$iptr <- 1
   e$iptr <- 1 + e$iptr
 }
 
 waitUser.text_order_question <- function(current.row, e){
   e$val <- str_trim(unlist(strsplit(readline("ANSWER: "),",")))
-#   e$row <- 1 + e$row
-#   e$iptr <- 1
   e$iptr <- 1 + e$iptr
 }
 
@@ -49,6 +43,7 @@ waitUser.video <- function(current.row, e){
   if(tolower(response) %in% c("y", "yes")){
     swirl_out("Type nxt() to continue")
     e$prompt <- TRUE
+    e$playing <- TRUE
     browseURL(current.row[,"VideoLink"])
   }
   e$row <- 1 + e$row
@@ -119,6 +114,12 @@ testResponse.default <- function(current.row, e){
     e$iptr <- 1
     e$row <- 1 + e$row
   } else {
+    # Restore the previous global environment from the snapshot
+    # in case the user has garbled it, e.g., has typed x <- 3*x
+    # instead of x <- 2*x by mistake. The hint might say to type
+    # x <- 2*x, which would result in 6 times the original value
+    # of x unless the original value is restored.
+    xfer(as.environment(e$snapshot), globalenv())
     mes <- tryAgain()
     if(is(current.row, "cmd_question")) {
       mes <- paste(mes, "Or, type info() for more options.")
@@ -131,9 +132,49 @@ testResponse.default <- function(current.row, e){
 }
 
 testMe <- function(keyphrase, e){
-  # Add a new class attribute to the keyphrase using
-  # the substring left of its first "=".
-  attr(keyphrase, "class") <- c(class(keyphrase),
-                                strsplit(keyphrase, "=")[[1]][1])
-  return(runTest(keyphrase, e))
+  # patch to accommodate old-style tests
+  oldcourse <- attr(e$les, "course_name") %in% 
+    c("Data Analysis", "Mathematical Biostatistics Boot Camp",
+      "Open Intro")
+  
+  if(oldcourse){
+    # Use old test syntax
+    # Add a new class attribute to the keyphrase using
+    # the substring left of its first "=".
+    attr(keyphrase, "class") <- c(class(keyphrase),
+                                  strsplit(keyphrase, "=")[[1]][1])
+    return(runTest(keyphrase, e))
+  } else {
+    # Use new test syntax
+    return(eval(parse(text=keyphrase)))
+  }
+}
+
+# CUSTOM TEST SUPPORT. An environment for custom tests is inserted
+# "between" function testMe and the swirl namespace. That is,  
+# an environment, customTests, is created with parent swirl
+# and child testMe. Code evaluated within testMe will thus search
+# for functions first in customTests, and then in the swirl namespace.
+#
+# Custom tests must be defined in a file named "customTests.R" in the
+# lesson directory. Tests in such files are loaded into environment 
+# customTests when a lesson is first loaded or progress is restored. 
+# The environment is cleared between lessons.
+
+# An environment with parent swirl to hold custom tests.
+customTests <- new.env(parent=environment(testMe))
+# Make customTests the parent of testMe.
+environment(testMe) <- customTests
+
+# Function to load custom tests from a source file.
+loadCustomTests <- function(lespath){
+  cfile <- file.path(lespath,"customTests.R")
+  if(file.exists(cfile)){
+    source(cfile, local=customTests)
+  }
+}
+
+# Function to remove everything from environment customTests
+clearCustomTests <- function(){
+  remove(list=ls(customTests), envir=customTests)
 }
